﻿using System;
using System.Collections.Generic;
using System.Net.Mail;
using System.Text;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using SelArom.Net.Sitefinity.API.Events;
using Telerik.Cms.Engine;
using Telerik.Cms.Web.UI;
using Telerik.Events;
using Telerik.Workflow;
using Telerik.Workflow.WebControls;
using Telerik;
using Telerik.Cms.Engine.Security;

namespace SelArom.Net.Sitefinity.Templates
{
	/// <summary>
	/// UserControl base for the EventsItemPreview external template to send notification emails for content status change
	/// </summary>
	public class EventsItemPreview : ContentItemPreviewBase
	{
		protected EventsManager eMgr;
		protected override void OnLoad(EventArgs e)
		{
			// initialize generic manager and local events manager
			DefaultProvider = EventsManager.DefaultContentProvider;
			mgr = new ContentManager(DefaultProvider);
			eMgr = new EventsManager(DefaultProvider);

			// initialize everything else (including content item data)
			base.OnLoad(e);

			// add custom handler for publishing
			if (wfButton == null) return;
			wfButton.Command += Clear_Cache;
		}

		protected override string ForApprovalMessage()
		{
			var sb = new StringBuilder();
			// create email body
			sb.AppendFormat("{0} has submitted an event item for approval!\n\nhttp://{1}/Sitefinity/Admin/Modules.aspx?module=Events\n\n", usr.UserName, Request.Url.Host);
			sb.Append("\nPlease REVIEW the entry and correct any issues before approving for publication!");

			// add event details if found
			if (contentItem != null)
			{
				sb.Append("Event Details:\n\n");

				// ensure a title
				var title = contentItem.GetMetaData("Title");
				if (title == null) title = contentItem.GetMetaData("Name");
				if (title == null) title = "N/A";

				sb.AppendFormat("Title: {0}\nStart: {1:MMM d, yyyy hh:mm tt}\nEnd: {2:MMM d, yyyy hh:mm tt}", title, contentItem.GetMetaData(eMgr.EventStartField), contentItem.GetMetaData(eMgr.EventEndField));
			}
			return sb.ToString();
		}

		protected override string ApprovedMessage()
		{
			var sb = new StringBuilder();
			// create email body
			sb.AppendFormat("{0} has approved an event item for publication!\n\nhttp://{1}/Sitefinity/Admin/Modules.aspx?module=Events\n\n", usr.UserName, Request.Url.Host);

			// add event details if found
			if (contentItem != null)
			{
				sb.Append("Event Details:\n\n");

				// ensure a title
				var title = contentItem.GetMetaData("Title");
				if (title == null) title = contentItem.GetMetaData("Name");
				if (title == null) title = "N/A";

				sb.AppendFormat("Title: {0}\nStart: {1:MMM d, yyyy hh:mm tt}\nEnd: {2:MMM d, yyyy hh:mm tt}", title, contentItem.GetMetaData(eMgr.EventStartField), contentItem.GetMetaData(eMgr.EventEndField));
			}
			return sb.ToString();
		}

		protected void Clear_Cache(object sender, CommandEventArgs e)
		{
			// make sure the command is already parsed
			if (string.IsNullOrEmpty(wfCommand)) return;

			// make sure content item was found
			if (contentItem == null) return;
			// clear cache on publish
			if (wfCommand.ToLower() == "publish")
			{
				// make sure meta field exists
				var meta = contentItem.GetMetaData(eMgr.EventStartField);
				if (meta == null) return;

				// make sure date is valid
				DateTime date;
				if (!DateTime.TryParse(meta.ToString(), out date)) return;

				// clear cache!
				var api = new EventsAPI();
				api.Clear(date);
			}
		}
	}
}